@@ -3,7 +3,7 @@
    Domain Name Service subroutines. */
 
 /*
- * Copyright (c) 2004-2006 by Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (c) 2004-2006,2009 by Internet Systems Consortium, Inc. ("ISC")
  * Copyright (c) 2001-2003 by Internet Software Consortium
  *
  * Permission to use, copy, modify, and distribute this software for any
@@ -22,18 +22,18 @@
  *   950 Charter Street
  *   Redwood City, CA 94063
  *   <info@isc.org>
- *   http://www.isc.org/
+ *   https://www.isc.org/
  *
  * This software has been written for Internet Systems Consortium
  * by Ted Lemon in cooperation with Nominum, Inc.
  * To learn more about Internet Systems Consortium, see
- * ``http://www.isc.org/''.  To learn more about Nominum, Inc., see
+ * ``https://www.isc.org/''.  To learn more about Nominum, Inc., see
  * ``http://www.nominum.com''.
  */
 
 #ifndef lint
 static char copyright[] =
-"$Id: dns.c,v 1.35.2.21 2006/02/22 22:43:27 dhankins Exp $ Copyright (c) 2004-2006 Internet Systems Consortium.  All rights reserved.\n";
+"$Id: dns.c,v 1.40.32.3 2009/09/01 20:32:27 dhankins Exp $ Copyright (c) 2004-2006,2009 Internet Systems Consortium.  All rights reserved.\n";
 #endif /* not lint */
 
 #include "dhcpd.h"
@@ -192,7 +192,7 @@ isc_result_t enter_dns_zone (struct dns_zone *zone)
 			dns_zone_dereference (&tz, MDL);
 		}
 	} else {
-		if (!dns_zone_new_hash (&dns_zone_hash, 1, MDL))
+		if (!dns_zone_new_hash(&dns_zone_hash, DNS_HASH_SIZE, MDL))
 			return ISC_R_NOMEMORY;
 	}
 	dns_zone_hash_add (dns_zone_hash, zone -> name, 0, zone, MDL);
@@ -518,10 +518,10 @@ int get_dhcid (struct data_string *id,
 /* Now for the DDNS update code that is shared between client and
    server... */
 
-isc_result_t ddns_update_a (struct data_string *ddns_fwd_name,
-			    struct iaddr ddns_addr,
-			    struct data_string *ddns_dhcid,
-			    unsigned long ttl, int rrsetp)
+isc_result_t
+ddns_update_a(struct data_string *ddns_fwd_name, struct iaddr ddns_addr,
+	      struct data_string *ddns_dhcid, unsigned long ttl,
+	      unsigned rrsetp, unsigned conflict)
 {
 	ns_updque updqueue;
 	ns_updrec *updrec;
@@ -654,23 +654,65 @@ isc_result_t ddns_update_a (struct data_string *ddns_fwd_name,
 		minires_freeupdrec (updrec);
 	}
 
-	/*
-	 * DHCID RR exists, and matches client identity.
+	/* If we're doing conflict resolution, we use a set of prereqs.  If
+	 * not, we delete the DHCID in addition to all A rrsets.
 	 */
-	updrec = minires_mkupdrec (S_PREREQ,
-				   (const char *)ddns_fwd_name -> data,
-				   C_IN, T_DHCID, 0);
-	if (!updrec) {
-		result = ISC_R_NOMEMORY;
-		goto error;
+	if (conflict) {
+		/*
+		 * DHCID RR exists, and matches client identity.
+		 */
+		updrec = minires_mkupdrec (S_PREREQ,
+					   (const char *)ddns_fwd_name -> data,
+					   C_IN, T_DHCID, 0);
+		if (!updrec) {
+			result = ISC_R_NOMEMORY;
+			goto error;
+		}
+	
+		updrec -> r_data = ddns_dhcid -> data;
+		updrec -> r_size = ddns_dhcid -> len;
+		updrec -> r_opcode = YXRRSET;
+
+		ISC_LIST_APPEND (updqueue, updrec, r_link);
+	} else {
+		/*
+		 * Conflict detection override: delete DHCID RRs.
+		 */
+		updrec = minires_mkupdrec(S_UPDATE, ddns_fwd_name->data,
+					  C_IN, T_DHCID, 0);
+
+		if (!updrec) {
+			result = ISC_R_NOMEMORY;
+			goto error;
+		}
+
+		updrec->r_data = NULL;
+		updrec->r_size = 0;
+		updrec->r_opcode = DELETE;
+
+		ISC_LIST_APPEND(updqueue, updrec, r_link);
+
+
+		/*
+		 * With all other DHCID RR's deleted, add this client's
+		 * DHCID unconditionally (as update-conflict-detection is
+		 * disabled).
+		 */
+		updrec = minires_mkupdrec(S_UPDATE,
+					  (const char *)ddns_fwd_name->data,
+					  C_IN, T_DHCID, ttl);
+		if (!updrec) {
+			result = ISC_R_NOMEMORY;
+			goto error;
+		}
+ 
+		updrec->r_data = ddns_dhcid->data;
+		updrec->r_size = ddns_dhcid->len;
+		updrec->r_opcode = ADD;
+ 
+		ISC_LIST_APPEND (updqueue, updrec, r_link);
 	}
 
-	updrec -> r_data = ddns_dhcid -> data;
-	updrec -> r_size = ddns_dhcid -> len;
-	updrec -> r_opcode = YXRRSET;
-
-	ISC_LIST_APPEND (updqueue, updrec, r_link);
-
 
 	/*
 	 * Delete A RRset.
@@ -963,4 +1005,4 @@ isc_result_t ddns_remove_a (struct data_string *ddns_fwd_name,
 #endif /* NSUPDATE */
 
 HASH_FUNCTIONS (dns_zone, const char *, struct dns_zone, dns_zone_hash_t,
-		dns_zone_reference, dns_zone_dereference)
+		dns_zone_reference, dns_zone_dereference, do_case_hash)
