@@ -3,7 +3,7 @@
    Turn data structures into printable text. */
 
 /*
- * Copyright (c) 2004 by Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (c) 2004-2007,2009 by Internet Systems Consortium, Inc. ("ISC")
  * Copyright (c) 1995-2003 by Internet Software Consortium
  *
  * Permission to use, copy, modify, and distribute this software for any
@@ -22,27 +22,29 @@
  *   950 Charter Street
  *   Redwood City, CA 94063
  *   <info@isc.org>
- *   http://www.isc.org/
+ *   https://www.isc.org/
  *
  * This software has been written for Internet Systems Consortium
  * by Ted Lemon in cooperation with Vixie Enterprises and Nominum, Inc.
  * To learn more about Internet Systems Consortium, see
- * ``http://www.isc.org/''.  To learn more about Vixie Enterprises,
+ * ``https://www.isc.org/''.  To learn more about Vixie Enterprises,
  * see ``http://www.vix.com''.   To learn more about Nominum, Inc., see
  * ``http://www.nominum.com''.
  */
 
 #ifndef lint
 static char copyright[] =
-"$Id: print.c,v 1.53.2.11 2004/06/17 20:54:39 dhankins Exp $ Copyright (c) 2004 Internet Systems Consortium.  All rights reserved.\n";
+"$Id: print.c,v 1.61.12.7 2009/09/01 20:32:27 dhankins Exp $ Copyright (c) 2004-2007,2009 Internet Systems Consortium.  All rights reserved.\n";
 #endif /* not lint */
 
 #include "dhcpd.h"
 
+int db_time_format = DEFAULT_TIME_FORMAT;
+
 char *quotify_string (const char *s, const char *file, int line)
 {
 	unsigned len = 0;
-	const char *sp;
+	const unsigned char *sp;
 	char *buf, *nsp;
 
 	for (sp = s; sp && *sp; sp++) {
@@ -459,7 +461,8 @@ static unsigned print_subexpression (expr, buf, len)
 {
 	unsigned rv, left;
 	const char *s;
-	
+	struct expression *next_arg;
+
 	switch (expr -> op) {
 	      case expr_none:
 		if (len > 3) {
@@ -550,6 +553,30 @@ static unsigned print_subexpression (expr, buf, len)
 		}
 		break;
 
+	      case expr_lcase:
+		if (len > 9) {
+			rv = 7;
+			strcpy(buf, "(lcase ");
+			rv += print_subexpression(expr->data.lcase,
+						  buf + rv, len - rv - 1);
+			buf[rv++] = ')';
+			buf[rv] = 0;
+			return rv;
+		}
+		break;
+
+	      case expr_ucase:
+		if (len > 9) {
+			rv = 7;
+			strcpy(buf, "(ucase ");
+			rv += print_subexpression(expr->data.ucase,
+						  buf + rv, len - rv - 1);
+			buf[rv++] = ')';
+			buf[rv] = 0;
+			return rv;
+		}
+		break;
+
 	      case expr_concat:
 		if (len > 10) {
 			rv = 8;
@@ -891,7 +918,7 @@ static unsigned print_subexpression (expr, buf, len)
 			rv += print_subexpression (expr -> data.b2a.width,
 						   buf + rv, len - rv - 3);
 			buf [rv++] = ' ';
-			rv += print_subexpression (expr -> data.b2a.seperator,
+			rv += print_subexpression (expr -> data.b2a.separator,
 						   buf + rv, len - rv - 2);
 			buf [rv++] = ' ';
 			rv += print_subexpression (expr -> data.b2a.buffer,
@@ -1007,6 +1034,7 @@ static unsigned print_subexpression (expr, buf, len)
 			return rv;
 		}
 		break;
+
 	      case expr_function:
 		rv = 9;
 		if (len > rv + 1) {
@@ -1020,10 +1048,15 @@ static unsigned print_subexpression (expr, buf, len)
 					rv += strlen (foo -> string);
 				}
 			}
-			buf [rv] = ')';
-			buf [rv++] = 0;
+			buf [rv++] = ')';
+			buf [rv] = 0;
 			return rv;
 		}
+
+	      default:
+		log_fatal("Impossible case at %s:%d (undefined expression "
+			  "%d).", MDL, expr->op);
+		break;
 	}
 	return 0;
 }
@@ -1067,6 +1100,7 @@ int token_print_indent_concat (FILE *file, int col,  int indent,
 		len = strlen (s);
 		strcpy (u, s);
 		u += len;
+		s = va_arg (list, char *);
 	}
 	va_end (list);
 	
@@ -1403,3 +1437,48 @@ void print_dns_status (int status, ns_updque *uq)
 		log_info ("%s", obuf);
 }
 #endif /* NSUPDATE */
+
+/* Format the given time as "A; # B", where A is the format
+ * used by the parser, and B is the local time, for humans.
+ */
+const char *
+print_time(TIME t)
+{
+	static char buf[sizeof("epoch 9223372036854775807; "
+			       "# Wed Jun 30 21:49:08 2147483647")];
+	/* The string: 	       "6 2147483647/12/31 23:59:60;"
+	 * is smaller than the other, used to declare the buffer size, so
+	 * we can use one buffer for both.
+	 */
+
+	if (t == MAX_TIME)
+		return "never;";
+
+	if (t < 0)
+		return NULL;
+
+	/* For those lucky enough to have a 128-bit time_t, ensure that
+	 * whatever (corrupt) value we're given doesn't exceed the static
+	 * buffer.
+	 */
+#if (MAX_TIME > 0x7fffffffffffffff)
+	if (t > 0x7fffffffffffffff)
+		return NULL;
+#endif
+
+	if (db_time_format == LOCAL_TIME_FORMAT) {
+		if (strftime(buf, sizeof(buf),
+			     "epoch %s; # %a %b %d %H:%M:%S %Y",
+			     localtime(&t)) == 0)
+			return NULL;
+	} else {
+		/* No bounds check for the year is necessary - in this case,
+		 * strftime() will run out of space and assert an error.
+		 */
+		if (strftime(buf, sizeof(buf), "%w %Y/%m/%d %H:%M:%S;",
+			     gmtime(&t)) == 0)
+			return NULL;
+	}
+
+	return buf;
+}
