@@ -51,11 +51,11 @@
  *   950 Charter Street
  *   Redwood City, CA 94063
  *   <info@isc.org>
- *   http://www.isc.org/
+ *   https://www.isc.org/
  */
 
 /*
- *	$Id: nameser.h,v 1.5 2005/03/17 20:15:10 dhankins Exp $
+ *	$Id: nameser.h,v 1.5.140.1 2009/07/23 21:43:34 sar Exp $
  */
 
 #ifndef _ARPA_NAMESER_H_
