@@ -3,7 +3,7 @@
    Routines for manipulating hash tables... */
 
 /*
- * Copyright (c) 2004 by Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (c) 2004-2008 by Internet Systems Consortium, Inc. ("ISC")
  * Copyright (c) 1995-2003 by Internet Software Consortium
  *
  * Permission to use, copy, modify, and distribute this software for any
@@ -22,34 +22,47 @@
  *   950 Charter Street
  *   Redwood City, CA 94063
  *   <info@isc.org>
- *   http://www.isc.org/
+ *   https://www.isc.org/
  *
  * This software has been written for Internet Systems Consortium
  * by Ted Lemon in cooperation with Vixie Enterprises and Nominum, Inc.
  * To learn more about Internet Systems Consortium, see
- * ``http://www.isc.org/''.  To learn more about Vixie Enterprises,
+ * ``https://www.isc.org/''.  To learn more about Vixie Enterprises,
  * see ``http://www.vix.com''.   To learn more about Nominum, Inc., see
  * ``http://www.nominum.com''.
  */
 
 #ifndef lint
 static char copyright[] =
-"$Id: hash.c,v 1.1.2.6 2004/06/10 17:59:47 dhankins Exp $ Copyright (c) 2004 Internet Systems Consortium.  All rights reserved.\n";
+"$Id: hash.c,v 1.11.16.6 2009/07/23 21:43:35 sar Exp $ Copyright (c) 2004-2008 Internet Systems Consortium.  All rights reserved.\n";
 #endif /* not lint */
 
 #include <omapip/omapip_p.h>
 #include <ctype.h>
 
-static int do_hash (const unsigned char *, unsigned, unsigned);
-static int do_case_hash (const unsigned char *, unsigned, unsigned);
+static INLINE unsigned
+find_length(const void *key,
+	    unsigned (*do_hash)(const void *, unsigned, unsigned))
+{
+	if (do_hash == do_case_hash || do_hash == do_string_hash)
+		return strlen((const char *)key);
+	if (do_hash == do_number_hash)
+		return sizeof(unsigned);
+	if (do_hash == do_ip4_hash)
+		return 4;
+
+	log_fatal("Impossible condition at %s:%d.", MDL);
+	return 0; /* Silence compiler warnings. */
+}
 
 int new_hash_table (tp, count, file, line)
 	struct hash_table **tp;
-	int count;
+	unsigned count;
 	const char *file;
 	int line;
 {
 	struct hash_table *rval;
+	unsigned extra;
 
 	if (!tp) {
 		log_error ("%s(%d): new_hash_table called with null pointer.",
@@ -66,9 +79,18 @@ int new_hash_table (tp, count, file, line)
 		abort ();
 #endif
 	}
-	rval = dmalloc (sizeof (struct hash_table) -
-			(DEFAULT_HASH_SIZE * sizeof (struct hash_bucket *)) +
-			(count * sizeof (struct hash_bucket *)), file, line);
+
+	/* There is one hash bucket in the structure.  Allocate extra
+	 * memory beyond the end of the structure to fulfill the requested
+	 * count ("count - 1").  Do not let there be less than one.
+	 */
+	if (count <= 1)
+		extra = 0;
+	else
+		extra = count - 1;
+
+	rval = dmalloc(sizeof(struct hash_table) +
+		       (extra * sizeof(struct hash_bucket *)), file, line);
 	if (!rval)
 		return 0;
 	rval -> hash_count = count;
@@ -188,42 +210,46 @@ void free_hash_bucket (ptr, file, line)
 	free_hash_buckets = ptr;
 }
 
-int new_hash (struct hash_table **rp,
-	      hash_reference referencer,
-	      hash_dereference dereferencer,
-	      int casep, const char *file, int line)
+int new_hash(struct hash_table **rp,
+	     hash_reference referencer,
+	     hash_dereference dereferencer,
+	     unsigned hsize,
+	     unsigned (*hasher)(const void *, unsigned, unsigned),
+	     const char *file, int line)
 {
-	if (!new_hash_table (rp, DEFAULT_HASH_SIZE, file, line))
+	if (hsize == 0)
+		hsize = DEFAULT_HASH_SIZE;
+
+	if (!new_hash_table (rp, hsize, file, line))
 		return 0;
-	memset (&(*rp) -> buckets [0], 0,
-		DEFAULT_HASH_SIZE * sizeof (struct hash_bucket *));
-	(*rp) -> referencer = referencer;
-	(*rp) -> dereferencer = dereferencer;
-	if (casep) {
-		(*rp) -> cmp = casecmp;
-		(*rp) -> do_hash = do_case_hash;
-	} else {
-		(*rp) -> cmp = (hash_comparator_t)memcmp;
-		(*rp) -> do_hash = do_hash;
-	}
+
+	memset ((*rp)->buckets, 0, hsize * sizeof(struct hash_bucket *));
+
+	(*rp)->referencer = referencer;
+	(*rp)->dereferencer = dereferencer;
+	(*rp)->do_hash = hasher;
+
+	if (hasher == do_case_hash)
+		(*rp)->cmp = casecmp;
+	else
+		(*rp)->cmp = memcmp;
+
 	return 1;
 }
 
-static int do_case_hash (name, len, size)
-	const unsigned char *name;
-	unsigned len;
-	unsigned size;
+unsigned
+do_case_hash(const void *name, unsigned len, unsigned size)
 {
-	register int accum = 0;
-	register const unsigned char *s = (const unsigned char *)name;
+	register unsigned accum = 0;
+	register const unsigned char *s = name;
 	int i = len;
 	register unsigned c;
 
 	while (i--) {
 		/* Make the hash case-insensitive. */
 		c = *s++;
-		if (isascii (c) && isupper (c))
-			c = tolower (c);
+		if (isascii(c))
+			c = tolower(c);
 
 		/* Add the character in... */
 		accum = (accum << 1) + c;
@@ -232,16 +258,15 @@ static int do_case_hash (name, len, size)
 		while (accum > 65535) {
 			accum = (accum & 65535) + (accum >> 16);
 		}
+
 	}
 	return accum % size;
 }
 
-static int do_hash (name, len, size)
-	const unsigned char *name;
-	unsigned len;
-	unsigned size;
+unsigned
+do_string_hash(const void *name, unsigned len, unsigned size)
 {
-	register int accum = 0;
+	register unsigned accum = 0;
 	register const unsigned char *s = (const unsigned char *)name;
 	int i = len;
 
@@ -257,10 +282,117 @@ static int do_hash (name, len, size)
 	return accum % size;
 }
 
-void add_hash (table, name, len, pointer, file, line)
+/* Client identifiers are generally 32-bits of ordinary
+ * non-randomness followed by 24-bits of unordinary randomness.
+ * So, end-align in 24-bit chunks, and xor any preceding data
+ * just to mix it up a little.
+ */
+unsigned
+do_id_hash(const void *name, unsigned len, unsigned size)
+{
+	register unsigned accum = 0;
+	register const unsigned char *s = (const unsigned char *)name;
+	const unsigned char *end = s + len;
+
+	if (len == 0)
+		return 0;
+
+	/* The switch indexes our starting position into the do/while loop,
+	 * taking up the remainder after hashing in all the other bytes in
+	 * threes.
+	 */
+	switch (len % 3) {
+		do {
+	      case 0:
+			accum ^= *s++ << 16;
+	      case 2:
+			accum ^= *s++ << 8;
+	      case 1:
+			accum ^= *s++;
+		} while (s < end);
+
+		break;
+	}
+
+	return accum % size;
+}
+
+unsigned
+do_number_hash(const void *key, unsigned len, unsigned size)
+{
+	register unsigned number = *((const unsigned *)key);
+
+	return number % size;
+}
+
+unsigned
+do_ip4_hash(const void *key, unsigned len, unsigned size)
+{
+	u_int32_t number;
+
+	memcpy(&number, key, 4);
+
+	number = ntohl(number);
+
+	return number % size;
+}
+
+unsigned char *
+hash_report(struct hash_table *table)
+{
+	static unsigned char retbuf[sizeof("Contents/Size (%): "
+					   "2147483647/2147483647 "
+					   "(2147483647%). "
+					   "Min/max: 2147483647/2147483647")];
+	unsigned curlen, pct, contents=0, minlen=UINT_MAX, maxlen=0;
+	unsigned i;
+	struct hash_bucket *bp;
+
+	if (table == NULL)
+		return (unsigned char *) "No table.";
+
+	if (table->hash_count == 0)
+		return (char *) "Invalid hash table.";
+
+	for (i = 0 ; i < table->hash_count ; i++) {
+		curlen = 0;
+
+		bp = table->buckets[i];
+		while (bp != NULL) {
+			curlen++;
+			bp = bp->next;
+		}
+
+		if (curlen < minlen)
+			minlen = curlen;
+		if (curlen > maxlen)
+			maxlen = curlen;
+
+		contents += curlen;
+	}
+
+	if (contents >= (UINT_MAX / 100))
+		pct = contents / ((table->hash_count / 100) + 1);
+	else
+		pct = (contents * 100) / table->hash_count;
+
+	if (contents > 2147483647 ||
+	    table->hash_count > 2147483647 ||
+	    pct > 2147483647 ||
+	    minlen > 2147483647 ||
+	    maxlen > 2147483647)
+		return (char *) "Report out of range for display.";
+
+	sprintf(retbuf, "Contents/Size (%%): %u/%u (%u%%). Min/max: %u/%u",
+		contents, table->hash_count, pct, minlen, maxlen);
+
+	return retbuf;
+}
+
+void add_hash (table, key, len, pointer, file, line)
 	struct hash_table *table;
 	unsigned len;
-	const unsigned char *name;
+	const void *key;
 	hashed_object_t *pointer;
 	const char *file;
 	int line;
@@ -273,16 +405,16 @@ void add_hash (table, name, len, pointer, file, line)
 		return;
 
 	if (!len)
-		len = strlen ((const char *)name);
+		len = find_length(key, table->do_hash);
 
-	hashno = (*table -> do_hash) (name, len, table -> hash_count);
+	hashno = (*table->do_hash)(key, len, table->hash_count);
 	bp = new_hash_bucket (file, line);
 
 	if (!bp) {
-		log_error ("Can't add %s to hash table.", name);
+		log_error ("Can't add entry to hash table: no memory.");
 		return;
 	}
-	bp -> name = name;
+	bp -> name = key;
 	if (table -> referencer) {
 		foo = &bp -> value;
 		(*(table -> referencer)) (foo, pointer, file, line);
@@ -293,10 +425,10 @@ void add_hash (table, name, len, pointer, file, line)
 	table -> buckets [hashno] = bp;
 }
 
-void delete_hash_entry (table, name, len, file, line)
+void delete_hash_entry (table, key, len, file, line)
 	struct hash_table *table;
 	unsigned len;
-	const unsigned char *name;
+	const void *key;
 	const char *file;
 	int line;
 {
@@ -308,17 +440,17 @@ void delete_hash_entry (table, name, len, file, line)
 		return;
 
 	if (!len)
-		len = strlen ((const char *)name);
+		len = find_length(key, table->do_hash);
 
-	hashno = (*table -> do_hash) (name, len, table -> hash_count);
+	hashno = (*table->do_hash)(key, len, table->hash_count);
 
 	/* Go through the list looking for an entry that matches;
 	   if we find it, delete it. */
 	for (bp = table -> buckets [hashno]; bp; bp = bp -> next) {
 		if ((!bp -> len &&
-		     !strcmp ((const char *)bp -> name, (const char *)name)) ||
+		     !strcmp ((const char *)bp->name, key)) ||
 		    (bp -> len == len &&
-		     !(*table -> cmp) (bp -> name, name, len))) {
+		     !(table -> cmp)(bp->name, key, len))) {
 			if (pbp) {
 				pbp -> next = bp -> next;
 			} else {
@@ -335,10 +467,10 @@ void delete_hash_entry (table, name, len, file, line)
 	}
 }
 
-int hash_lookup (vp, table, name, len, file, line)
+int hash_lookup (vp, table, key, len, file, line)
 	hashed_object_t **vp;
 	struct hash_table *table;
-	const unsigned char *name;
+	const void *key;
 	unsigned len;
 	const char *file;
 	int line;
@@ -349,13 +481,18 @@ int hash_lookup (vp, table, name, len, file, line)
 	if (!table)
 		return 0;
 	if (!len)
-		len = strlen ((const char *)name);
+		len = find_length(key, table->do_hash);
 
-	hashno = (*table -> do_hash) (name, len, table -> hash_count);
+	if (*vp != NULL) {
+		log_fatal("Internal inconsistency: storage value has not been "
+			  "initialized to zero (from %s:%d).", file, line);
+	}
+
+	hashno = (*table->do_hash)(key, len, table->hash_count);
 
 	for (bp = table -> buckets [hashno]; bp; bp = bp -> next) {
 		if (len == bp -> len
-		    && !(*table -> cmp) (bp -> name, name, len)) {
+		    && !(*table->cmp)(bp->name, key, len)) {
 			if (table -> referencer)
 				(*table -> referencer) (vp, bp -> value,
 							file, line);
@@ -380,7 +517,9 @@ int hash_foreach (struct hash_table *table, hash_foreach_func func)
 		bp = table -> buckets [i];
 		while (bp) {
 			next = bp -> next;
-			(*func) (bp -> name, bp -> len, bp -> value);
+			if ((*func)(bp->name, bp->len, bp->value)
+							!= ISC_R_SUCCESS)
+				return count;
 			bp = next;
 			count++;
 		}
@@ -388,25 +527,25 @@ int hash_foreach (struct hash_table *table, hash_foreach_func func)
 	return count;
 }
 
-int casecmp (const void *v1, const void *v2, unsigned long len)
+int casecmp (const void *v1, const void *v2, size_t len)
 {
-	unsigned i;
-	const char *s = v1;
-	const char *t = v2;
+	size_t i;
+	const unsigned char *s = v1;
+	const unsigned char *t = v2;
 	
 	for (i = 0; i < len; i++)
 	{
 		int c1, c2;
-		if (isascii (s [i]) && isupper (s [i]))
-			c1 = tolower (s [i]);
+		if (isascii(s[i]))
+			c1 = tolower(s[i]);
 		else
-			c1 = s [i];
-		
-		if (isascii (t [i]) && isupper (t [i]))
-			c2 = tolower (t [i]);
+			c1 = s[i];
+
+		if (isascii(t[i]))
+			c2 = tolower(t[i]);
 		else
-			c2 = t [i];
-		
+			c2 = t[i];
+
 		if (c1 < c2)
 			return -1;
 		if (c1 > c2)
