@@ -3,7 +3,7 @@
    Persistent database management routines for DHCPD... */
 
 /*
- * Copyright (c) 2004-2005 by Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (c) 2004-2006 by Internet Systems Consortium, Inc. ("ISC")
  * Copyright (c) 1995-2003 by Internet Software Consortium
  *
  * Permission to use, copy, modify, and distribute this software for any
@@ -22,19 +22,19 @@
  *   950 Charter Street
  *   Redwood City, CA 94063
  *   <info@isc.org>
- *   http://www.isc.org/
+ *   https://www.isc.org/
  *
  * This software has been written for Internet Systems Consortium
  * by Ted Lemon in cooperation with Vixie Enterprises and Nominum, Inc.
  * To learn more about Internet Systems Consortium, see
- * ``http://www.isc.org/''.  To learn more about Vixie Enterprises,
+ * ``https://www.isc.org/''.  To learn more about Vixie Enterprises,
  * see ``http://www.vix.com''.   To learn more about Nominum, Inc., see
  * ``http://www.nominum.com''.
  */
 
 #ifndef lint
 static char copyright[] =
-"$Id: db.c,v 1.63.2.14 2005/10/14 16:02:55 dhankins Exp $ Copyright (c) 2004-2005 Internet Systems Consortium.  All rights reserved.\n";
+"$Id: db.c,v 1.74.32.5 2009/07/23 21:43:35 sar Exp $ Copyright (c) 2004-2006 Internet Systems Consortium.  All rights reserved.\n";
 #endif /* not lint */
 
 #include "dhcpd.h"
@@ -53,12 +53,11 @@ int lease_file_is_corrupt = 0;
 int write_lease (lease)
 	struct lease *lease;
 {
-	struct tm *t;
-	char tbuf [64];
 	int errors = 0;
 	int i;
 	struct binding *b;
 	char *s;
+	const char *tval;
 
 	/* If the lease file is corrupt, don't try to write any more leases
 	   until we've written a good lease file. */
@@ -72,110 +71,61 @@ int write_lease (lease)
 	fprintf (db_file, "lease %s {", piaddr (lease -> ip_addr));
 	if (errno) {
 		++errors;
-	}
+        }
 
-	/* Note: the following is not a Y2K bug - it's a Y1.9K bug.   Until
-	   somebody invents a time machine, I think we can safely disregard
-	   it. */
-	if (lease -> starts) {
-		if (lease -> starts != MAX_TIME) {
-			t = gmtime (&lease -> starts);
-			/* %Audit% Cannot exceed 59 bytes. %2004.06.17,Safe% */
-			sprintf (tbuf, "%d %d/%02d/%02d %02d:%02d:%02d;",
-				 t -> tm_wday, t -> tm_year + 1900,
-				 t -> tm_mon + 1, t -> tm_mday,
-				 t -> tm_hour, t -> tm_min, t -> tm_sec);
-		} else
-			strcpy (tbuf, "never;");
-		errno = 0;
-		fprintf (db_file, "\n  starts %s", tbuf);
-		if (errno) {
-			++errors;
-		}
-	}
+	if (lease->starts &&
+	    ((tval = print_time(lease->starts)) == NULL ||
+	     fprintf(db_file, "\n  starts %s", tval) < 0))
+		++errors;
 
-	if (lease -> ends) {
-		if (lease -> ends != MAX_TIME) {
-			t = gmtime (&lease -> ends);
-			/* %Audit% Cannot exceed 59 bytes. %2004.06.17,Safe% */
-			sprintf (tbuf, "%d %d/%02d/%02d %02d:%02d:%02d;",
-				 t -> tm_wday, t -> tm_year + 1900,
-				 t -> tm_mon + 1, t -> tm_mday,
-				 t -> tm_hour, t -> tm_min, t -> tm_sec);
-		} else
-			strcpy (tbuf, "never;");
-		errno = 0;
-		fprintf (db_file, "\n  ends %s", tbuf);
-		if (errno) {
-			++errors;
-		}
-	}
+	if (lease->ends &&
+	    ((tval = print_time(lease->ends)) == NULL ||
+	     fprintf(db_file, "\n  ends %s", tval) < 0))
+		++errors;
 
-	if (lease -> tstp) {
-		t = gmtime (&lease -> tstp);
-		errno = 0;
-		fprintf (db_file, "\n  tstp %d %d/%02d/%02d %02d:%02d:%02d;",
-			 t -> tm_wday, t -> tm_year + 1900,
-			 t -> tm_mon + 1, t -> tm_mday,
-			 t -> tm_hour, t -> tm_min, t -> tm_sec);
-		if (errno) {
-			++errors;
-		}
-	}
-	if (lease -> tsfp) {
-		t = gmtime (&lease -> tsfp);
-		errno = 0;
-		fprintf (db_file, "\n  tsfp %d %d/%02d/%02d %02d:%02d:%02d;",
-			 t -> tm_wday, t -> tm_year + 1900,
-			 t -> tm_mon + 1, t -> tm_mday,
-			 t -> tm_hour, t -> tm_min, t -> tm_sec);
-		if (errno) {
-			++errors;
-		}
-	}
-	if (lease->atsfp) {
-		t = gmtime(&lease->atsfp);
-		if (fprintf(db_file,
-			    "\n  atsfp %d %d/%02d/%02d %02d:%02d:%02d;",
-			    t->tm_wday, t->tm_year + 1900, t->tm_mon + 1,
-			    t->tm_mday, t->tm_hour, t->tm_min, t->tm_sec) <= 0)
-			++errors;
-	}
-	if (lease -> cltt) {
-		t = gmtime (&lease -> cltt);
-		errno = 0;
-		fprintf (db_file, "\n  cltt %d %d/%02d/%02d %02d:%02d:%02d;",
-			 t -> tm_wday, t -> tm_year + 1900,
-			 t -> tm_mon + 1, t -> tm_mday,
-			 t -> tm_hour, t -> tm_min, t -> tm_sec);
-		if (errno) {
-			++errors;
-		}
-	}
+	if (lease->tstp &&
+	    ((tval = print_time(lease->tstp)) == NULL ||
+	     fprintf(db_file, "\n  tstp %s", tval) < 0))
+		++errors;
 
-	if (lease -> binding_state == FTS_ACTIVE &&
-	    (lease -> flags & BOOTP_LEASE)) {
-		fprintf (db_file, "\n  binding state bootp;\n");
-	} else {
-		fprintf (db_file, "\n  binding state %s;",
-			 ((lease -> binding_state > 0 &&
-			   lease -> binding_state <= FTS_LAST)
-			  ? binding_state_names [lease -> binding_state - 1]
-			  : "abandoned"));
-	}
+	if (lease->tsfp &&
+	    ((tval = print_time(lease->tsfp)) == NULL ||
+	     fprintf(db_file, "\n  tsfp %s", tval) < 0))
+		++errors;
 
-	if (lease -> binding_state != lease -> next_binding_state) {
-	    if (lease -> next_binding_state == FTS_ACTIVE &&
-		(lease -> flags & BOOTP_LEASE))
-		fprintf (db_file, "\n  next binding state bootp;\n");
-	    else
-		fprintf (db_file, "\n  next binding state %s;",
+	if (lease->atsfp &&
+	    ((tval = print_time(lease->atsfp)) == NULL ||
+	     fprintf(db_file, "\n  atsfp %s", tval) < 0))
+		++errors;
+
+	if (lease->cltt &&
+	    ((tval = print_time(lease->cltt)) == NULL ||
+	     fprintf(db_file, "\n  cltt %s", tval) < 0))
+		++errors;
+
+	if (fprintf (db_file, "\n  binding state %s;",
+		 ((lease -> binding_state > 0 &&
+		   lease -> binding_state <= FTS_LAST)
+		  ? binding_state_names [lease -> binding_state - 1]
+		  : "abandoned")) < 0)
+                ++errors;
+
+	if (lease -> binding_state != lease -> next_binding_state)
+		if (fprintf (db_file, "\n  next binding state %s;",
 			 ((lease -> next_binding_state > 0 &&
 			   lease -> next_binding_state <= FTS_LAST)
 			  ? (binding_state_names
 			     [lease -> next_binding_state - 1])
-		  : "abandoned"));
-	}
+			  : "abandoned")) < 0)
+                        ++errors;
+
+	if (lease->flags & RESERVED_LEASE)
+		if (fprintf(db_file, "\n  reserved;") < 0)
+                        ++errors;
+
+	if (lease->flags & BOOTP_LEASE)
+		if (fprintf(db_file, "\n  dynamic-bootp;") < 0)
+                        ++errors;
 
 	/* If this lease is billed to a class and is still valid,
 	   write it out. */
@@ -194,14 +144,14 @@ int write_lease (lease)
 			 print_hw_addr (lease -> hardware_addr.hbuf [0],
 					lease -> hardware_addr.hlen - 1,
 					&lease -> hardware_addr.hbuf [1]));
-		if (errno) {
+		if (errno)
 			++errors;
-		}
 	}
 	if (lease -> uid_len) {
 		int i;
 		s = quotify_buf (lease -> uid, lease -> uid_len, MDL);
 		if (s) {
+			errno = 0;
 			fprintf (db_file, "\n  uid \"%s\";", s);
 			if (errno)
 				++errors;
@@ -288,31 +238,33 @@ int write_lease (lease)
 		fprintf (db_file, "\n  on expiry%s {",
 			 lease -> on_expiry == lease -> on_release
 			 ? " or release" : "");
-		if (errno)
-			++errors;
 		write_statements (db_file, lease -> on_expiry, 4);
 		/* XXX */
 		fprintf (db_file, "\n  }");
+		if (errno)
+			++errors;
 	}
 	if (lease -> on_release && lease -> on_release != lease -> on_expiry) {
 		errno = 0;
 		fprintf (db_file, "\n  on release {");
-		if (errno)
-			++errors;
 		write_statements (db_file, lease -> on_release, 4);
 		/* XXX */
 		fprintf (db_file, "\n  }");
+		if (errno)
+			++errors;
 	}
+
 	errno = 0;
 	fputs ("\n}\n", db_file);
-	if (errno) {
+	if (errno)
 		++errors;
-	}
-	if (errors)
+
+	if (errors) {
 		log_info ("write_lease: unable to write lease %s",
 		      piaddr (lease -> ip_addr));
-	if (errors)
 		lease_file_is_corrupt = 1;
+        }
+
 	return !errors;
 }
 
@@ -334,12 +286,11 @@ int write_host (host)
 
 	if (counting)
 		++count;
-	errno = 0;
 
+	errno = 0;
 	fprintf (db_file, "host %s {", host -> name);
-	if (errno) {
+	if (errno)
 		++errors;
-	}
 
 	if (host -> flags & HOST_DECL_DYNAMIC) {
 		errno = 0;
@@ -361,9 +312,8 @@ int write_host (host)
 				 print_hw_addr (host -> interface.hbuf [0],
 						host -> interface.hlen - 1,
 						&host -> interface.hbuf [1]));
-			if (errno) {
+			if (errno)
 				++errors;
-			}
 		}
 		if (host -> client_identifier.len) {
 			int i;
@@ -373,24 +323,28 @@ int write_host (host)
 				fprintf (db_file, "\n  uid \"%.*s\";",
 					 (int)host -> client_identifier.len,
 					 host -> client_identifier.data);
+				if (errno)
+					++errors;
 			} else {
 				fprintf (db_file,
 					 "\n  uid %2.2x",
 					 host -> client_identifier.data [0]);
-				if (errno) {
+				if (errno)
 					++errors;
-				}
 				for (i = 1;
 				     i < host -> client_identifier.len; i++) {
 					errno = 0;
 					fprintf (db_file, ":%2.2x",
 						 host ->
 						 client_identifier.data [i]);
-					if (errno) {
+					if (errno)
 						++errors;
-					}
 				}
-				putc (';', db_file);
+
+                                errno = 0;
+				fputc (';', db_file);
+				if (errno)
+					++errors;
 			}
 		}
 		
@@ -406,10 +360,10 @@ int write_host (host)
 		
 			errno = 0;
 			fprintf (db_file, "\n  fixed-address ");
-			if (errno) {
+			if (errno)
 				++errors;
-			}
 			for (i = 0; i < ip_addrs.len - 3; i += 4) {
+
 				errno = 0;
 				fprintf (db_file, "%u.%u.%u.%u%s",
 					 ip_addrs.data [i] & 0xff,
@@ -417,24 +371,22 @@ int write_host (host)
 					 ip_addrs.data [i + 2] & 0xff,
 					 ip_addrs.data [i + 3] & 0xff,
 					 i + 7 < ip_addrs.len ? "," : "");
-				if (errno) {
+				if (errno)
 					++errors;
-				}
 			}
+
 			errno = 0;
 			fputc (';', db_file);
-			if (errno) {
+			if (errno)
 				++errors;
-			}
 		}
 
 		if (host -> named_group) {
 			errno = 0;
 			fprintf (db_file, "\n  group \"%s\";",
 				 host -> named_group -> name);
-			if (errno) {
+			if (errno)
 				++errors;
-			}
 		}
 
 		if (host -> group &&
@@ -444,22 +396,22 @@ int write_host (host)
 			errno = 0;
 			write_statements (db_file,
 					  host -> group -> statements, 8);
-			if (errno) {
+			if (errno)
 				++errors;
-			}
 		}
 	}
 
 	errno = 0;
 	fputs ("\n}\n", db_file);
-	if (errno) {
+	if (errno)
 		++errors;
-	}
+
 	if (errors) {
 		log_info ("write_host: unable to write host %s",
 			  host -> name);
 		lease_file_is_corrupt = 1;
 	}
+
 	return !errors;
 }
 
@@ -480,12 +432,11 @@ int write_group (group)
 
 	if (counting)
 		++count;
-	errno = 0;
 
+	errno = 0;
 	fprintf (db_file, "group %s {", group -> name);
-	if (errno) {
+	if (errno)
 		++errors;
-	}
 
 	if (group -> flags & GROUP_OBJECT_DYNAMIC) {
 		errno = 0;
@@ -511,22 +462,22 @@ int write_group (group)
 			errno = 0;
 			write_statements (db_file,
 					  group -> group -> statements, 8);
-			if (errno) {
+			if (errno)
 				++errors;
-			}
 		}
 	}
 
 	errno = 0;
 	fputs ("\n}\n", db_file);
-	if (errno) {
+	if (errno)
 		++errors;
-	}
+
 	if (errors) {
 		log_info ("write_group: unable to write group %s",
 			  group -> name);
 		lease_file_is_corrupt = 1;
 	}
+
 	return !errors;
 }
 
@@ -535,6 +486,7 @@ int write_failover_state (dhcp_failover_state_t *state)
 {
 	struct tm *t;
 	int errors = 0;
+	const char *tval;
 
 	if (lease_file_is_corrupt)
 		if (!new_lease_file ())
@@ -545,28 +497,20 @@ int write_failover_state (dhcp_failover_state_t *state)
 	if (errno)
 		++errors;
 
-	t = gmtime (&state -> me.stos);
-	errno = 0;
-	fprintf (db_file, "\n  my state %s at %d %d/%02d/%02d %02d:%02d:%02d;",
-		 /* Never record our state as "startup"! */
-		 (state -> me.state == startup
-		  ? dhcp_failover_state_name_print (state -> saved_state)
-		  : dhcp_failover_state_name_print (state -> me.state)),
-		 t -> tm_wday, t -> tm_year + 1900,
-		 t -> tm_mon + 1, t -> tm_mday,
-		 t -> tm_hour, t -> tm_min, t -> tm_sec);
-	if (errno)
+	tval = print_time(state->me.stos);
+	if (tval == NULL ||
+	    fprintf(db_file, "\n  my state %s at %s",
+		    (state->me.state == startup) ?
+		    dhcp_failover_state_name_print(state->saved_state) :
+		    dhcp_failover_state_name_print(state->me.state),
+		    tval) < 0)
 		++errors;
 
-	t = gmtime (&state -> partner.stos);
-	errno = 0;
-	fprintf (db_file,
-		 "\n  partner state %s at %d %d/%02d/%02d %02d:%02d:%02d;",
-		 dhcp_failover_state_name_print (state -> partner.state),
-		 t -> tm_wday, t -> tm_year + 1900,
-		 t -> tm_mon + 1, t -> tm_mday,
-		 t -> tm_hour, t -> tm_min, t -> tm_sec);
-	if (errno)
+	tval = print_time(state->partner.stos);
+	if (tval == NULL ||
+	    fprintf(db_file, "\n  partner state %s at %s",
+		    dhcp_failover_state_name_print(state->partner.state),
+		    tval) < 0)
 		++errors;
 
 	if (state -> i_am == secondary) {
@@ -576,6 +520,8 @@ int write_failover_state (dhcp_failover_state_t *state)
 		if (errno)
 			++errors;
 	}
+
+        errno = 0;
 	fprintf (db_file, "\n}\n");
 	if (errno)
 		++errors;
@@ -586,13 +532,14 @@ int write_failover_state (dhcp_failover_state_t *state)
 		lease_file_is_corrupt = 1;
 		return 0;
 	}
+
 	return 1;
 
 }
 #endif
 
 int db_printable (s)
-	const char *s;
+	const unsigned char *s;
 {
 	int i;
 	for (i = 0; s [i]; i++)
@@ -607,6 +554,7 @@ int db_printable_len (s, len)
 	unsigned len;
 {
 	int i;
+
 	for (i = 0; i < len; i++)
 		if (!isascii (s [i]) || !isprint (s [i]) ||
 		    s [i] == '"' || s [i] == '\\')
@@ -614,11 +562,134 @@ int db_printable_len (s, len)
 	return 1;
 }
 
-void write_named_billing_class (const char *name, unsigned len,
-				struct class *class)
+static int print_hash_string(FILE *fp, struct class *class)
 {
-	/* XXX billing classes that are modified by OMAPI need
-	   XXX to be detected and written out here. */
+	int i;
+
+	for (i = 0 ; i < class->hash_string.len ; i++)
+		if (!isascii(class->hash_string.data[i]) ||
+		    !isprint(class->hash_string.data[i]))
+			break;
+
+	if (i == class->hash_string.len) {
+		if (fprintf(fp, " \"%.*s\"", (int)class->hash_string.len,
+			    class->hash_string.data) <= 0) {
+			log_error("Failure writing hash string: %m");
+			return 0;
+		}
+	} else {
+		if (fprintf(fp, " %2.2x", class->hash_string.data[0]) <= 0) {
+			log_error("Failure writing hash string: %m");
+			return 0;
+		}
+		for (i = 1 ; i < class->hash_string.len ; i++) {
+			if (fprintf(fp, ":%2.2x",
+				    class->hash_string.data[i]) <= 0) {
+				log_error("Failure writing hash string: %m");
+				return 0;
+			}
+		}
+	}
+
+	return 1;
+}
+
+
+isc_result_t
+write_named_billing_class(const void *key, unsigned len, void *object)
+{
+	const unsigned char *name = key;
+	struct class *class = object;
+
+	if (class->flags & CLASS_DECL_DYNAMIC) {
+		numclasseswritten++;
+		if (class->superclass == 0) {
+			if (fprintf(db_file, "class \"%s\" {\n", name) <= 0)
+				return ISC_R_IOERROR;
+		} else {
+			if (fprintf(db_file, "subclass \"%s\"",
+				    class->superclass->name) <= 0)
+				return ISC_R_IOERROR;
+			if (!print_hash_string(db_file, class))
+				return ISC_R_IOERROR;
+			if (fprintf(db_file, " {\n") <= 0)
+				return ISC_R_IOERROR;
+		}
+
+		if ((class->flags & CLASS_DECL_DELETED) != 0) {
+			if (fprintf(db_file, "  deleted;\n") <= 0)
+				return ISC_R_IOERROR;
+		} else {
+			if (fprintf(db_file, "  dynamic;\n") <= 0)
+				return ISC_R_IOERROR;
+		}
+	
+		if (class->lease_limit > 0) {
+			if (fprintf(db_file, "  lease limit %d;\n",
+				    class->lease_limit) <= 0)
+				return ISC_R_IOERROR;
+		}
+
+		if (class->expr != 0) {
+			if (fprintf(db_file, "  match if ") <= 0)
+				return ISC_R_IOERROR;
+
+                        errno = 0;                                       
+			write_expression(db_file, class->expr, 5, 5, 0);
+                        if (errno)
+                                return ISC_R_IOERROR;
+
+			if (fprintf(db_file, ";\n") <= 0)
+				return ISC_R_IOERROR;
+		}
+
+		if (class->submatch != 0) {
+			if (class->spawning) {
+				if (fprintf(db_file, "  spawn ") <= 0)
+					return ISC_R_IOERROR;
+			} else {
+				if (fprintf(db_file, "  match ") <= 0)
+					return ISC_R_IOERROR;
+			}
+
+                        errno = 0;
+			write_expression(db_file, class->submatch, 5, 5, 0);
+                        if (errno)
+                                return ISC_R_IOERROR;
+
+			if (fprintf(db_file, ";\n") <= 0)
+				return ISC_R_IOERROR;
+		}
+	
+		if (class->statements != 0) {
+                        errno = 0;
+			write_statements(db_file, class->statements, 8);
+                        if (errno)
+                                return ISC_R_IOERROR;
+		}
+
+		/* XXXJAB this isn't right, but classes read in off the
+		   leases file don't get the root group assigned to them
+		   (due to clone_group() call). */
+		if (class->group != 0 && class->group->authoritative != 0) {
+                        errno = 0;
+			write_statements(db_file, class->group->statements, 8);
+                        if (errno)
+                                return ISC_R_IOERROR;
+                }
+
+		if (fprintf(db_file, "}\n\n") <= 0)
+			return ISC_R_IOERROR;
+	}
+
+	if (class->hash != NULL) {	/* yep. recursive. god help us. */
+		/* XXX - cannot check error status of this...
+		 * foo_hash_foreach returns a count of operations completed.
+		 */
+		class_hash_foreach(class->hash, write_named_billing_class);
+	}
+
+	return ISC_R_SUCCESS;
 }
 
 void write_billing_classes ()
@@ -655,44 +726,20 @@ int write_billing_class (class)
 		return !errno;
 	}
 
-	errno = 0;
-	fprintf (db_file, "\n  billing subclass \"%s\"",
-		 class -> superclass -> name);
-	if (errno)
+	if (fprintf(db_file, "\n  billing subclass \"%s\"",
+		    class -> superclass -> name) < 0)
 		++errors;
 
-	for (i = 0; i < class -> hash_string.len; i++)
-		if (!isascii (class -> hash_string.data [i]) ||
-		    !isprint (class -> hash_string.data [i]))
-			break;
-	if (i == class -> hash_string.len) {
-		errno = 0;
-		fprintf (db_file, " \"%.*s\";",
-			 (int)class -> hash_string.len,
-			 class -> hash_string.data);
-		if (errno)
-			++errors;
-	} else {
-		errno = 0;
-		fprintf (db_file, " %2.2x", class -> hash_string.data [0]);
-		if (errno)
-			++errors;
-		for (i = 1; i < class -> hash_string.len; i++) {
-			errno = 0;
-			fprintf (db_file, ":%2.2x",
-				 class -> hash_string.data [i]);
-			if (errno)
-				++errors;
-		}
-		errno = 0;
-		fprintf (db_file, ";");
-		if (errno)
-			++errors;
-	}
+	if (!print_hash_string(db_file, class))
+                ++errors;
+
+	if (fprintf(db_file, ";") < 0)
+                ++errors;
 
 	class -> dirty = !errors;
 	if (errors)
 		lease_file_is_corrupt = 1;
+
 	return !errors;
 }
 
@@ -766,6 +813,16 @@ void db_startup (testp)
 			GET_TIME (&write_time);
 		new_lease_file ();
 	}
+
+#if defined(REPORT_HASH_PERFORMANCE)
+	log_info("Host HW hash:   %s", host_hash_report(host_hw_addr_hash));
+	log_info("Host UID hash:  %s", host_hash_report(host_uid_hash));
+	log_info("Lease IP hash:  %s",
+		 lease_ip_hash_report(lease_ip_addr_hash));
+	log_info("Lease UID hash: %s", lease_id_hash_report(lease_uid_hash));
+	log_info("Lease HW hash:  %s",
+		 lease_id_hash_report(lease_hw_addr_hash));
+#endif
 }
 
 int new_lease_file ()
@@ -774,11 +831,14 @@ int new_lease_file ()
 	char backfname [512];
 	TIME t;
 	int db_fd;
+	int db_validity;
 	FILE *new_db_file;
 
 	/* Make a temporary lease file... */
 	GET_TIME (&t);
 
+	db_validity = lease_file_is_corrupt;
+
 	/* %Audit% Truncated filename causes panic. %2004.06.17,Safe%
 	 * This should never happen since the path is a configuration
 	 * variable from build-time or command-line.  But if it should,
@@ -805,37 +865,22 @@ int new_lease_file ()
 		fclose(db_file);
 	db_file = new_db_file;
 
-	/* Write an introduction so people don't complain about time
-	   being off. */
 	errno = 0;
-	fprintf (db_file, "# All times in this file are in UTC (GMT), not %s",
-		 "your local timezone.   This is\n");
-	if (errno != 0)
-		goto fail;
-	fprintf (db_file, "# not a bug, so please don't ask about it.   %s",
-		 "There is no portable way to\n");
-	if (errno != 0)
-		goto fail;
-	fprintf (db_file, "# store leases in the local timezone, so please %s",
-		 "don't request this as a\n");
-	if (errno != 0)
-		goto fail;
-	fprintf (db_file, "# feature.   If this is inconvenient or %s",
-		 "confusing to you, we sincerely\n");
-	if (errno != 0)
-		goto fail;
-	fprintf (db_file, "# apologize.   Seriously, though - don't ask.\n");
-	if (errno != 0)
-		goto fail;
 	fprintf (db_file, "# The format of this file is documented in the %s",
 		 "dhcpd.leases(5) manual page.\n");
-	if (errno != 0)
+	if (errno)
 		goto fail;
+
 	fprintf (db_file, "# This lease file was written by isc-dhcp-%s\n\n",
 		 DHCP_VERSION);
-	if (errno != 0)
+	if (errno)
 		goto fail;
 
+	/* At this point we have a new lease file that, so far, could not
+	 * be described as either corrupt nor valid.
+	 */
+	lease_file_is_corrupt = 0;
+
 	/* Write out all the leases that we know of... */
 	counting = 0;
 	if (!write_leases ())
@@ -882,11 +927,10 @@ int new_lease_file ()
 	}
 
 	counting = 1;
-	lease_file_is_corrupt = 0;
 	return 1;
 
       fail:
-	lease_file_is_corrupt = 1;
+	lease_file_is_corrupt = db_validity;
       fdfail:
 	unlink (newfname);
 	return 0;
