@@ -3,7 +3,7 @@
    Server-specific in-memory database support. */
 
 /*
- * Copyright (c) 2004-2007 by Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (c) 2004-2009 by Internet Systems Consortium, Inc. ("ISC")
  * Copyright (c) 1996-2003 by Internet Software Consortium
  *
  * Permission to use, copy, modify, and distribute this software for any
@@ -22,19 +22,19 @@
  *   950 Charter Street
  *   Redwood City, CA 94063
  *   <info@isc.org>
- *   http://www.isc.org/
+ *   https://www.isc.org/
  *
  * This software has been written for Internet Systems Consortium
  * by Ted Lemon in cooperation with Vixie Enterprises and Nominum, Inc.
  * To learn more about Internet Systems Consortium, see
- * ``http://www.isc.org/''.  To learn more about Vixie Enterprises,
+ * ``https://www.isc.org/''.  To learn more about Vixie Enterprises,
  * see ``http://www.vix.com''.   To learn more about Nominum, Inc., see
  * ``http://www.nominum.com''.
  */
 
 #ifndef lint
 static char copyright[] =
-"$Id: mdb.c,v 1.67.2.31 2007/06/08 18:59:13 dhankins Exp $ Copyright (c) 2004-2005 Internet Systems Consortium.  All rights reserved.\n";
+"$Id: mdb.c,v 1.83.16.15 2009/09/01 20:32:28 dhankins Exp $ Copyright (c) 2004-2008 Internet Systems Consortium.  All rights reserved.\n";
 #endif /* not lint */
 
 #include "dhcpd.h"
@@ -45,12 +45,65 @@ struct shared_network *shared_networks;
 host_hash_t *host_hw_addr_hash;
 host_hash_t *host_uid_hash;
 host_hash_t *host_name_hash;
-lease_hash_t *lease_uid_hash;
-lease_hash_t *lease_ip_addr_hash;
-lease_hash_t *lease_hw_addr_hash;
+lease_id_hash_t *lease_uid_hash;
+lease_ip_hash_t *lease_ip_addr_hash;
+lease_id_hash_t *lease_hw_addr_hash;
+
+int numclasseswritten;
 
 omapi_object_type_t *dhcp_type_host;
 
+isc_result_t enter_class(cd, dynamicp, commit)
+	struct class *cd;
+	int dynamicp;
+	int commit;
+{
+	if (!collections -> classes) {
+		/* A subclass with no parent is invalid. */
+		if (cd->name == NULL)
+			return ISC_R_INVALIDARG;
+
+		class_reference (&collections -> classes, cd, MDL);
+	} else if (cd->name != NULL) {	/* regular class */
+		struct class *c = 0;
+
+		if (find_class(&c, cd->name, MDL) != ISC_R_NOTFOUND) {
+			class_dereference(&c, MDL);
+			return ISC_R_EXISTS;
+		}
+		
+		/* Find the tail. */
+		for (c = collections -> classes;
+		     c -> nic; c = c -> nic)
+			/* nothing */ ;
+		class_reference (&c -> nic, cd, MDL);
+	}
+
+	if (dynamicp && commit) {
+		const char *name = cd->name;
+
+		if (name == NULL) {
+			name = cd->superclass->name;
+		}
+
+		write_named_billing_class ((const unsigned char *)name, 0, cd);
+		if (!commit_leases ())
+			return ISC_R_IOERROR;
+	}
+
+	return ISC_R_SUCCESS;
+}
+
+
+/* Variable to check if we're starting the server.  The server will init as
+ * starting - but just to be safe start out as false to avoid triggering new
+ * special-case code
+ * XXX: There is actually a server_startup state...which is never entered...
+ */
+#define SS_NOSYNC	1
+#define SS_QFOLLOW	2
+static int server_starting = 0;
+
 static int find_uid_statement (struct executable_statement *esp,
 			       void *vp, int condp)
 {
@@ -86,7 +139,7 @@ isc_result_t enter_host (hd, dynamicp, commit)
 	struct executable_statement *esp;
 
 	if (!host_name_hash) {
-		if (!host_new_hash (&host_name_hash, 0, MDL))
+		if (!host_new_hash(&host_name_hash, HOST_HASH_SIZE, MDL))
 			log_fatal ("Can't allocate host name hash");
 		host_hash_add (host_name_hash,
 			       (unsigned char *)hd -> name,
@@ -141,7 +194,8 @@ isc_result_t enter_host (hd, dynamicp, commit)
 
 	if (hd -> interface.hlen) {
 		if (!host_hw_addr_hash) {
-			if (!host_new_hash (&host_hw_addr_hash, 0, MDL))
+			if (!host_new_hash(&host_hw_addr_hash,
+					   HOST_HASH_SIZE, MDL))
 				log_fatal ("Can't allocate host/hw hash");
 		} else {
 			/* If there isn't already a host decl matching this
@@ -185,7 +239,8 @@ isc_result_t enter_host (hd, dynamicp, commit)
 		/* If there's no uid hash, make one; otherwise, see if
 		   there's already an entry in the hash for this host. */
 		if (!host_uid_hash) {
-			if (!host_new_hash (&host_uid_hash, 0, MDL))
+			if (!host_new_hash(&host_uid_hash,
+					   HOST_HASH_SIZE, MDL))
 				log_fatal ("Can't allocate host/uid hash");
 
 			host_hash_add (host_uid_hash,
@@ -231,6 +286,30 @@ isc_result_t enter_host (hd, dynamicp, commit)
 	return ISC_R_SUCCESS;
 }
 
+
+isc_result_t delete_class (cp, commit)
+	struct class *cp;
+	int commit;
+{
+	cp->flags |= CLASS_DECL_DELETED;
+
+	/* do the write first as we won't be leaving it in any data
+	   structures, unlike the host objects */
+	
+	if (commit) {
+		write_named_billing_class ((unsigned char *)cp->name, 0, cp);
+		if (!commit_leases ())
+			return ISC_R_IOERROR;
+	}
+	
+	unlink_class(&cp);		/* remove from collections */
+
+	class_dereference(&cp, MDL);
+
+	return ISC_R_SUCCESS;
+}
+
+
 isc_result_t delete_host (hd, commit)
 	struct host_decl *hd;
 	int commit;
@@ -453,8 +532,10 @@ void new_address_range (cfile, low, high, subnet, pool, lpchain)
 	struct pool *pool;
 	struct lease **lpchain;
 {
-	struct lease *address_range, *lp, *plp;
-	struct iaddr net;
+#if defined(COMPACT_LEASES)
+	struct lease *address_range;
+#endif
+	struct lease *lp, *plp;
 	unsigned min, max, i;
 	char lowbuf [16], highbuf [16], netbuf [16];
 	struct shared_network *share = subnet -> shared_network;
@@ -470,35 +551,33 @@ void new_address_range (cfile, low, high, subnet, pool, lpchain)
 
 	/* Initialize the hash table if it hasn't been done yet. */
 	if (!lease_uid_hash) {
-		if (!lease_new_hash (&lease_uid_hash, 0, MDL))
+		if (!lease_id_new_hash(&lease_uid_hash, LEASE_HASH_SIZE, MDL))
 			log_fatal ("Can't allocate lease/uid hash");
 	}
 	if (!lease_ip_addr_hash) {
-		if (!lease_new_hash (&lease_ip_addr_hash, 0, MDL))
+		if (!lease_ip_new_hash(&lease_ip_addr_hash, LEASE_HASH_SIZE,
+				       MDL))
 			log_fatal ("Can't allocate lease/ip hash");
 	}
 	if (!lease_hw_addr_hash) {
-		if (!lease_new_hash (&lease_hw_addr_hash, 0, MDL))
+		if (!lease_id_new_hash(&lease_hw_addr_hash, LEASE_HASH_SIZE,
+				       MDL))
 			log_fatal ("Can't allocate lease/hw hash");
 	}
 
-	/* Make sure that high and low addresses are in same subnet. */
-	net = subnet_number (low, subnet -> netmask);
-	if (!addr_eq (net, subnet_number (high, subnet -> netmask))) {
-		strcpy (lowbuf, piaddr (low));
-		strcpy (highbuf, piaddr (high));
-		strcpy (netbuf, piaddr (subnet -> netmask));
-		log_fatal ("Address range %s to %s, netmask %s spans %s!",
-		       lowbuf, highbuf, netbuf, "multiple subnets");
+	/* Make sure that high and low addresses are in this subnet. */
+	if (!addr_eq(subnet->net, subnet_number(low, subnet->netmask))) {
+		strcpy(lowbuf, piaddr(low));
+		strcpy(netbuf, piaddr(subnet->net));
+		log_fatal("bad range, address %s not in subnet %s netmask %s",
+			  lowbuf, netbuf, piaddr(subnet->netmask));
 	}
 
-	/* Make sure that the addresses are on the correct subnet. */
-	if (!addr_eq (net, subnet -> net)) {
-		strcpy (lowbuf, piaddr (low));
-		strcpy (highbuf, piaddr (high));
-		strcpy (netbuf, piaddr (subnet -> netmask));
-		log_fatal ("Address range %s to %s not on net %s/%s!",
-		       lowbuf, highbuf, piaddr (subnet -> net), netbuf);
+	if (!addr_eq(subnet->net, subnet_number(high, subnet->netmask))) {
+		strcpy(highbuf, piaddr(high));
+		strcpy(netbuf, piaddr(subnet->net));
+		log_fatal("bad range, address %s not in subnet %s netmask %s",
+			  highbuf, netbuf, piaddr(subnet->netmask));
 	}
 
 	/* Get the high and low host addresses... */
@@ -541,7 +620,7 @@ void new_address_range (cfile, low, high, subnet, pool, lpchain)
 #endif
 		lp -> ip_addr = ip_addr (subnet -> net,
 					 subnet -> netmask, i + min);
-		lp -> starts = lp -> timestamp = MIN_TIME;
+		lp -> starts = MIN_TIME;
 		lp -> ends = MIN_TIME;
 		subnet_reference (&lp -> subnet, subnet, MDL);
 		pool_reference (&lp -> pool, pool, MDL);
@@ -559,9 +638,9 @@ void new_address_range (cfile, low, high, subnet, pool, lpchain)
 				pool_reference (&lt -> pool, pool, MDL);
 			lease_dereference (&lt, MDL);
 		} else
-			lease_hash_add (lease_ip_addr_hash,
-					lp -> ip_addr.iabuf,
-					lp -> ip_addr.len, lp, MDL);
+			lease_ip_hash_add(lease_ip_addr_hash,
+					  lp->ip_addr.iabuf, lp->ip_addr.len,
+					  lp, MDL);
 		/* Put the lease on the chain for the caller. */
 		if (lpchain) {
 			if (*lpchain) {
@@ -757,9 +836,9 @@ void enter_lease (lease)
 		if (comp -> subnet)
 			subnet_reference (&lease -> subnet,
 					  comp -> subnet, MDL);
-		lease_hash_delete (lease_ip_addr_hash,
-				   lease -> ip_addr.iabuf,
-				   lease -> ip_addr.len, MDL);
+		lease_ip_hash_delete(lease_ip_addr_hash,
+				     lease->ip_addr.iabuf, lease->ip_addr.len,
+				     MDL);
 		lease_dereference (&comp, MDL);
 	}
 
@@ -776,9 +855,8 @@ void enter_lease (lease)
 		log_error ("lease %s: no subnet.", piaddr (lease -> ip_addr));
 		return;
 	}
-	lease_hash_add (lease_ip_addr_hash,
-			lease -> ip_addr.iabuf,
-			lease -> ip_addr.len, lease, MDL);
+	lease_ip_hash_add(lease_ip_addr_hash, lease->ip_addr.iabuf,
+			  lease->ip_addr.len, lease, MDL);
 }
 
 /* Replace the data in an existing lease with the data in a new lease;
@@ -794,8 +872,9 @@ int supersede_lease (comp, lease, commit, propogate, pimmediate)
 {
 	struct lease *lp, **lq, *prev;
 	TIME lp_next_state;
-
 #if defined (FAILOVER_PROTOCOL)
+	int do_pool_check = 0;
+
 	/* We must commit leases before sending updates regarding them
 	   to failover peers.  It is, therefore, an error to set pimmediate
 	   and not commit. */
@@ -949,8 +1028,8 @@ int supersede_lease (comp, lease, commit, propogate, pimmediate)
 	if (comp->hardware_addr.hlen)
 		hw_hash_add(comp);
 
-#if defined (FAILOVER_PROTOCOL)
 	comp->cltt = lease->cltt;
+#if defined (FAILOVER_PROTOCOL)
 	comp->tstp = lease->tstp;
 	comp->tsfp = lease->tsfp;
 	comp->atsfp = lease->atsfp;
@@ -977,8 +1056,16 @@ int supersede_lease (comp, lease, commit, propogate, pimmediate)
 	/* Figure out which queue it's on. */
 	switch (comp -> binding_state) {
 	      case FTS_FREE:
-		lq = &comp -> pool -> free;
-		comp -> pool -> free_leases--;
+		if (comp->flags & RESERVED_LEASE)
+			lq = &comp->pool->reserved;
+		else {
+			lq = &comp->pool->free;
+			comp->pool->free_leases--;
+		}
+
+#if defined(FAILOVER_PROTOCOL)
+		do_pool_check = 1;
+#endif
 		break;
 
 	      case FTS_ACTIVE:
@@ -996,8 +1083,16 @@ int supersede_lease (comp, lease, commit, propogate, pimmediate)
 		break;
 
 	      case FTS_BACKUP:
-		lq = &comp -> pool -> backup;
-		comp -> pool -> backup_leases--;
+		if (comp->flags & RESERVED_LEASE)
+			lq = &comp->pool->reserved;
+		else {
+			lq = &comp->pool->backup;
+			comp->pool->backup_leases--;
+		}
+
+#if defined(FAILOVER_PROTOCOL)
+		do_pool_check = 1;
+#endif
 		break;
 
 	      default:
@@ -1076,8 +1171,10 @@ int supersede_lease (comp, lease, commit, propogate, pimmediate)
 	if (commit) {
 		if (!write_lease (comp))
 			return 0;
-		if (!commit_leases ())
-			return 0;
+		if ((server_starting & SS_NOSYNC) == 0) {
+			if (!commit_leases ())
+				return 0;
+		}
 	}
 
 #if defined (FAILOVER_PROTOCOL)
@@ -1086,6 +1183,8 @@ int supersede_lease (comp, lease, commit, propogate, pimmediate)
 		if (!dhcp_failover_queue_update (comp, pimmediate))
 			return 0;
 	}
+	if (do_pool_check && comp->pool->failover_peer)
+		dhcp_failover_pool_check(comp->pool);
 #endif
 
 	/* If the current binding state has already expired, do an
@@ -1128,6 +1227,21 @@ void make_binding_state_transition (struct lease *lease)
 	      lease -> binding_state == FTS_ACTIVE &&
 	      lease -> next_binding_state != FTS_RELEASED))) {
 #if defined (NSUPDATE)
+		/*
+		 * Note: ddns_removals() is also iterated when the lease
+		 * enters state 'released' in 'release_lease()'.  The below
+		 * is caught when a peer receives a BNDUPD from a failover
+		 * peer; it may not have received the client's release (it
+		 * may have been offline).
+		 *
+		 * We could remove the call from release_lease() because
+		 * it will also catch here on the originating server after the
+		 * peer acknowledges the state change.  However, there could
+		 * be many hours inbetween, and in this case we /know/ the
+		 * client is no longer using the lease when we receive the
+		 * release message.  This is not true of expiry, where the
+		 * peer may have extended the lease.
+		 */
 		ddns_removals (lease);
 #endif
 		if (lease -> on_expiry) {
@@ -1279,7 +1393,6 @@ int lease_copy (struct lease **lp,
 	lt -> ip_addr = lease -> ip_addr;
 	lt -> starts = lease -> starts;
 	lt -> ends = lease -> ends;
-	lt -> timestamp = lease -> timestamp;
 	lt -> uid_len = lease -> uid_len;
 	lt -> uid_max = lease -> uid_max;
 	if (lease -> uid == lease -> uid_buf) {
@@ -1378,7 +1491,14 @@ void release_lease (lease, packet)
 		/* Blow away any bindings. */
 		if (lease -> scope)
 			binding_scope_dereference (&lease -> scope, MDL);
+
+		/* Set sort times to the present. */
 		lease -> ends = cur_time;
+		/* Lower layers of muckery set tstp to ->ends.  But we send
+		 * protocol messages before this.  So it is best to set
+		 * tstp now anyway.
+		 */
+		lease->tstp = cur_time;
 #if defined (FAILOVER_PROTOCOL)
 		if (lease -> pool && lease -> pool -> failover_peer) {
 			lease -> next_binding_state = FTS_RELEASED;
@@ -1467,24 +1587,26 @@ void pool_timer (vpool)
 	struct lease *lt = (struct lease *)0;
 	struct lease *next = (struct lease *)0;
 	struct lease *lease = (struct lease *)0;
-	struct lease **lptr [5];
+#define FREE_LEASES 0
+#define ACTIVE_LEASES 1
+#define EXPIRED_LEASES 2
+#define ABANDONED_LEASES 3
+#define BACKUP_LEASES 4
+#define RESERVED_LEASES 5
+	struct lease **lptr[RESERVED_LEASES+1];
 	TIME next_expiry = MAX_TIME;
 	int i;
 
 	pool = (struct pool *)vpool;
 
-#define FREE_LEASES 0
 	lptr [FREE_LEASES] = &pool -> free;
-#define ACTIVE_LEASES 1
 	lptr [ACTIVE_LEASES] = &pool -> active;
-#define EXPIRED_LEASES 2
 	lptr [EXPIRED_LEASES] = &pool -> expired;
-#define ABANDONED_LEASES 3
 	lptr [ABANDONED_LEASES] = &pool -> abandoned;
-#define BACKUP_LEASES 4
 	lptr [BACKUP_LEASES] = &pool -> backup;
+	lptr[RESERVED_LEASES] = &pool->reserved;
 
-	for (i = FREE_LEASES; i <= BACKUP_LEASES; i++) {
+	for (i = FREE_LEASES; i <= RESERVED_LEASES; i++) {
 		/* If there's nothing on the queue, skip it. */
 		if (!*(lptr [i]))
 			continue;
@@ -1555,8 +1677,8 @@ void pool_timer (vpool)
 int find_lease_by_ip_addr (struct lease **lp, struct iaddr addr,
 			   const char *file, int line)
 {
-	return lease_hash_lookup (lp, lease_ip_addr_hash,
-				  addr.iabuf, addr.len, file, line);
+	return lease_ip_hash_lookup(lp, lease_ip_addr_hash, addr.iabuf,
+				    addr.len, file, line);
 }
 
 int find_lease_by_uid (struct lease **lp, const unsigned char *uid,
@@ -1564,7 +1686,7 @@ int find_lease_by_uid (struct lease **lp, const unsigned char *uid,
 {
 	if (len == 0)
 		return 0;
-	return lease_hash_lookup (lp, lease_uid_hash, uid, len, file, line);
+	return lease_id_hash_lookup (lp, lease_uid_hash, uid, len, file, line);
 }
 
 int find_lease_by_hw_addr (struct lease **lp,
@@ -1573,8 +1695,8 @@ int find_lease_by_hw_addr (struct lease **lp,
 {
 	if (hwlen == 0)
 		return 0;
-	return lease_hash_lookup (lp, lease_hw_addr_hash,
-				  hwaddr, hwlen, file, line);
+	return lease_id_hash_lookup(lp, lease_hw_addr_hash, hwaddr, hwlen,
+				    file, line);
 }
 
 /* If the lease is preferred over the candidate, return truth.  The
@@ -1635,8 +1757,8 @@ uid_hash_add(struct lease *lease)
 
 	/* If it's not in the hash, just add it. */
 	if (!find_lease_by_uid (&head, lease -> uid, lease -> uid_len, MDL))
-		lease_hash_add (lease_uid_hash, lease -> uid,
-				lease -> uid_len, lease, MDL);
+		lease_id_hash_add(lease_uid_hash, lease->uid, lease->uid_len,
+				  lease, MDL);
 	else {
 		/* Otherwise, insert it into the list in order of its
 		 * preference for "resuming allocation to the client."
@@ -1670,10 +1792,10 @@ uid_hash_add(struct lease *lease)
 		 */
 		if (prev == NULL) {
 			lease_reference(&lease->n_uid, head, MDL);
-			lease_hash_delete(lease_uid_hash, lease->uid,
-					  lease->uid_len, MDL);
-			lease_hash_add(lease_uid_hash, lease->uid,
-				       lease->uid_len, lease, MDL);
+			lease_id_hash_delete(lease_uid_hash, lease->uid,
+					     lease->uid_len, MDL);
+			lease_id_hash_add(lease_uid_hash, lease->uid,
+					  lease->uid_len, lease, MDL);
 		} else /* (prev != NULL) */ {
 			if(prev->n_uid != NULL) {
 				lease_reference(&lease->n_uid, prev->n_uid,
@@ -1710,13 +1832,12 @@ void uid_hash_delete (lease)
 	   remove the hash table entry and add a new one with the
 	   next lease on the list (if there is one). */
 	if (head == lease) {
-		lease_hash_delete (lease_uid_hash,
-				   lease -> uid, lease -> uid_len, MDL);
+		lease_id_hash_delete(lease_uid_hash, lease->uid,
+				     lease->uid_len, MDL);
 		if (lease -> n_uid) {
-			lease_hash_add (lease_uid_hash,
-					lease -> n_uid -> uid,
-					lease -> n_uid -> uid_len,
-					lease -> n_uid, MDL);
+			lease_id_hash_add(lease_uid_hash, lease->n_uid->uid,
+					  lease->n_uid->uid_len, lease->n_uid,
+					  MDL);
 			lease_dereference (&lease -> n_uid, MDL);
 		}
 	} else {
@@ -1752,10 +1873,9 @@ hw_hash_add(struct lease *lease)
 	/* If it's not in the hash, just add it. */
 	if (!find_lease_by_hw_addr (&head, lease -> hardware_addr.hbuf,
 				    lease -> hardware_addr.hlen, MDL))
-		lease_hash_add (lease_hw_addr_hash,
-				lease -> hardware_addr.hbuf,
-				lease -> hardware_addr.hlen,
-				lease, MDL);
+		lease_id_hash_add(lease_hw_addr_hash,
+				  lease->hardware_addr.hbuf,
+				  lease->hardware_addr.hlen, lease, MDL);
 	else {
 		/* Otherwise, insert it into the list in order of its
 		 * preference for "resuming allocation to the client."
@@ -1789,13 +1909,13 @@ hw_hash_add(struct lease *lease)
 		 */
 		if (prev == NULL) {
 			lease_reference(&lease->n_hw, head, MDL);
-			lease_hash_delete(lease_hw_addr_hash,
+			lease_id_hash_delete(lease_hw_addr_hash,
+					     lease->hardware_addr.hbuf,
+					     lease->hardware_addr.hlen, MDL);
+			lease_id_hash_add(lease_hw_addr_hash,
 					  lease->hardware_addr.hbuf,
-					  lease->hardware_addr.hlen, MDL);
-			lease_hash_add(lease_hw_addr_hash,
-				       lease->hardware_addr.hbuf,
-				       lease->hardware_addr.hlen,
-				       lease, MDL);
+					  lease->hardware_addr.hlen,
+					  lease, MDL);
 		} else /* (prev != NULL) */ {
 			if(prev->n_hw != NULL) {
 				lease_reference(&lease->n_hw, prev->n_hw,
@@ -1833,15 +1953,15 @@ void hw_hash_delete (lease)
 	   remove the hash table entry and add a new one with the
 	   next lease on the list (if there is one). */
 	if (head == lease) {
-		lease_hash_delete (lease_hw_addr_hash,
-				   lease -> hardware_addr.hbuf,
-				   lease -> hardware_addr.hlen, MDL);
-		if (lease -> n_hw) {
-			lease_hash_add (lease_hw_addr_hash,
-					lease -> n_hw -> hardware_addr.hbuf,
-					lease -> n_hw -> hardware_addr.hlen,
-					lease -> n_hw, MDL);
-			lease_dereference (&lease -> n_hw, MDL);
+		lease_id_hash_delete(lease_hw_addr_hash,
+				     lease->hardware_addr.hbuf,
+				     lease->hardware_addr.hlen, MDL);
+		if (lease->n_hw) {
+			lease_id_hash_add(lease_hw_addr_hash,
+					  lease->n_hw->hardware_addr.hbuf,
+					  lease->n_hw->hardware_addr.hlen,
+					  lease->n_hw, MDL);
+			lease_dereference(&lease->n_hw, MDL);
 		}
 	} else {
 		/* Otherwise, look for the lease in the list of leases
@@ -1878,10 +1998,29 @@ int write_leases ()
 	struct host_decl *hp;
 	struct group_object *gp;
 	struct hash_bucket *hb;
+	struct class *cp;
+	struct collection *colp;
 	int i;
 	int num_written;
-	struct lease **lptr [5];
+	struct lease **lptr[RESERVED_LEASES+1];
 
+	/* write all the dynamically-created class declarations. */
+	if (collections->classes) {
+		numclasseswritten = 0;
+		for (colp = collections ; colp ; colp = colp->next) {
+			for (cp = colp->classes ; cp ; cp = cp->nic) {
+				write_named_billing_class(
+						(unsigned char *)cp->name,
+							  0, cp);
+			}
+		}
+
+		/* XXXJAB this number doesn't include subclasses... */ 
+		log_info ("Wrote %d class decls to leases file.",
+			  numclasseswritten);
+	}
+	
+			
 	/* Write all the dynamically-created group declarations. */
 	if (group_name_hash) {
 	    num_written = 0;
@@ -1952,8 +2091,9 @@ int write_leases ()
 		lptr [EXPIRED_LEASES] = &p -> expired;
 		lptr [ABANDONED_LEASES] = &p -> abandoned;
 		lptr [BACKUP_LEASES] = &p -> backup;
+		lptr [RESERVED_LEASES] = &p->reserved;
 
-		for (i = FREE_LEASES; i <= BACKUP_LEASES; i++) {
+		for (i = FREE_LEASES; i <= RESERVED_LEASES; i++) {
 		    for (l = *(lptr [i]); l; l = l -> next) {
 #if !defined (DEBUG_DUMP_ALL_LEASES)
 			if (l -> hardware_addr.hlen ||
@@ -1990,6 +2130,8 @@ int write_leases ()
 int lease_enqueue (struct lease *comp)
 {
 	struct lease **lq, *prev, *lp;
+	static struct lease **last_lq = NULL;
+	static struct lease *last_insert_point = NULL;
 
 	/* No queue to put it on? */
 	if (!comp -> pool)
@@ -1998,8 +2140,12 @@ int lease_enqueue (struct lease *comp)
 	/* Figure out which queue it's going to. */
 	switch (comp -> binding_state) {
 	      case FTS_FREE:
-		lq = &comp -> pool -> free;
-		comp -> pool -> free_leases++;
+		if (comp->flags & RESERVED_LEASE) {
+			lq = &comp->pool->reserved;
+		} else {
+			lq = &comp->pool->free;
+			comp->pool->free_leases++;
+		}
 		comp -> sort_time = comp -> ends;
 		break;
 
@@ -2041,8 +2187,12 @@ int lease_enqueue (struct lease *comp)
 		break;
 
 	      case FTS_BACKUP:
-		lq = &comp -> pool -> backup;
-		comp -> pool -> backup_leases++;
+		if (comp->flags & RESERVED_LEASE) {
+			lq = &comp->pool->reserved;
+		} else {
+			lq = &comp->pool->backup;
+			comp->pool->backup_leases++;
+		}
 		comp -> sort_time = comp -> ends;
 		break;
 
@@ -2055,13 +2205,28 @@ int lease_enqueue (struct lease *comp)
 		return 0;
 	}
 
+	/* This only works during server startup: during runtime, the last
+	 * lease may be dequeued inbetween calls.  If the queue is the same
+	 * as was used previously, and the lease structure isn't (this is not
+	 * a re-queue), use that as a starting point for the insertion-sort.
+	 */
+	if ((server_starting & SS_QFOLLOW) && (lq == last_lq) &&
+	    (comp != last_insert_point) && 
+	    (last_insert_point->sort_time <= comp->sort_time)) {
+		prev = last_insert_point;
+		lp = prev->next;
+	} else {
+		prev = NULL;
+		lp = *lq;
+	}
+
 	/* Insertion sort the lease onto the appropriate queue. */
-	prev = (struct lease *)0;
-	for (lp = *lq; lp; lp = lp -> next) {
+	for (; lp ; lp = lp->next) {
 		if (lp -> sort_time >= comp -> sort_time)
 			break;
 		prev = lp;
 	}
+
 	if (prev) {
 		if (prev -> next) {
 			lease_reference (&comp -> next, prev -> next, MDL);
@@ -2075,6 +2240,8 @@ int lease_enqueue (struct lease *comp)
 		}
 		lease_reference (lq, comp, MDL);
 	}
+	last_insert_point = comp;
+	last_lq = lq;
 	return 1;
 }
 
@@ -2082,33 +2249,39 @@ int lease_enqueue (struct lease *comp)
    in each appropriate hash, understanding that it's already by definition
    in lease_ip_addr_hash. */
 
-void lease_instantiate (const unsigned char *val, unsigned len,
-			struct lease *lease)
+isc_result_t
+lease_instantiate(const void *key, unsigned len, void *object)
 {
+	struct lease *lease = object;
 	struct class *class;
 	/* XXX If the lease doesn't have a pool at this point, it's an
 	   XXX orphan, which we *should* keep around until it expires,
 	   XXX but which right now we just forget. */
 	if (!lease -> pool) {
-		lease_hash_delete (lease_ip_addr_hash,
-				   lease -> ip_addr.iabuf,
-				   lease -> ip_addr.len, MDL);
-		return;
+		lease_ip_hash_delete(lease_ip_addr_hash, lease->ip_addr.iabuf,
+				     lease->ip_addr.len, MDL);
+		return ISC_R_SUCCESS;
 	}
 		
-	/* Put the lease on the right queue. */
-	lease_enqueue (lease);
+	/* Put the lease on the right queue.  Failure to queue is probably
+	 * due to a bogus binding state.  In such a case, we claim success,
+	 * so that later leases in a hash_foreach are processed, but we
+	 * return early as we really don't want hw address hash entries or
+	 * other cruft to surround such a bogus entry.
+	 */
+	if (!lease_enqueue(lease))
+		return ISC_R_SUCCESS;
 
 	/* Record the lease in the uid hash if possible. */
 	if (lease -> uid) {
 		uid_hash_add (lease);
 	}
-	
+
 	/* Record it in the hardware address hash if possible. */
 	if (lease -> hardware_addr.hlen) {
 		hw_hash_add (lease);
 	}
-	
+
 	/* If the lease has a billing class, set up the billing. */
 	if (lease -> billing_class) {
 		class = (struct class *)0;
@@ -2123,7 +2296,7 @@ void lease_instantiate (const unsigned char *val, unsigned len,
 			bill_class (lease, class);
 		class_dereference (&class, MDL);
 	}
-	return;
+	return ISC_R_SUCCESS;
 }
 
 /* Run expiry events on every pool.   This is called on startup so that
@@ -2138,14 +2311,22 @@ void expire_all_pools ()
 	struct hash_bucket *hb;
 	int i;
 	struct lease *l;
-	struct lease **lptr [5];
+	struct lease **lptr[RESERVED_LEASES+1];
+
+	/* Indicate that we are in the startup phase */
+	server_starting = SS_NOSYNC | SS_QFOLLOW;
 
 	/* First, go over the hash list and actually put all the leases
 	   on the appropriate lists. */
-	lease_hash_foreach (lease_ip_addr_hash, lease_instantiate);
+	lease_ip_hash_foreach(lease_ip_addr_hash, lease_instantiate);
 
 	/* Loop through each pool in each shared network and call the
-	   expiry routine on the pool. */
+	 * expiry routine on the pool.  It is no longer safe to follow
+	 * the queue insertion point, as expiration of a lease can move
+	 * it between queues (and this may be the lease that function
+	 * points at).
+	 */
+	server_starting &= ~SS_QFOLLOW;
 	for (s = shared_networks; s; s = s -> next) {
 	    for (p = s -> pools; p; p = p -> next) {
 		pool_timer (p);
@@ -2153,27 +2334,28 @@ void expire_all_pools ()
 		p -> lease_count = 0;
 		p -> free_leases = 0;
 		p -> backup_leases = 0;
-		
+
 		lptr [FREE_LEASES] = &p -> free;
 		lptr [ACTIVE_LEASES] = &p -> active;
 		lptr [EXPIRED_LEASES] = &p -> expired;
 		lptr [ABANDONED_LEASES] = &p -> abandoned;
 		lptr [BACKUP_LEASES] = &p -> backup;
+		lptr [RESERVED_LEASES] = &p->reserved;
 
-		for (i = FREE_LEASES; i <= BACKUP_LEASES; i++) {
+		for (i = FREE_LEASES; i <= RESERVED_LEASES; i++) {
 		    for (l = *(lptr [i]); l; l = l -> next) {
 			p -> lease_count++;
 			if (l -> ends <= cur_time) {
 				if (l->binding_state == FTS_FREE) {
 					if (i == FREE_LEASES)
 						p->free_leases++;
-					else
+					else if (i != RESERVED_LEASES)
 						log_fatal("Impossible case "
 							  "at %s:%d.", MDL);
 				} else if (l->binding_state == FTS_BACKUP) {
 					if (i == BACKUP_LEASES)
 						p->backup_leases++;
-					else
+					else if (i != RESERVED_LEASES)
 						log_fatal("Impossible case "
 							  "at %s:%d.", MDL);
 				}
@@ -2190,6 +2372,9 @@ void expire_all_pools ()
 		}
 	    }
 	}
+
+	/* turn off startup phase */
+	server_starting = 0;
 }
 
 void dump_subnets ()
@@ -2198,7 +2383,7 @@ void dump_subnets ()
 	struct shared_network *s;
 	struct subnet *n;
 	struct pool *p;
-	struct lease **lptr [5];
+	struct lease **lptr[RESERVED_LEASES+1];
 	int i;
 
 	log_info ("Subnets:");
@@ -2216,8 +2401,9 @@ void dump_subnets ()
 		lptr [EXPIRED_LEASES] = &p -> expired;
 		lptr [ABANDONED_LEASES] = &p -> abandoned;
 		lptr [BACKUP_LEASES] = &p -> backup;
+		lptr [RESERVED_LEASES] = &p->reserved;
 
-		for (i = FREE_LEASES; i <= BACKUP_LEASES; i++) {
+		for (i = FREE_LEASES; i <= RESERVED_LEASES; i++) {
 		    for (l = *(lptr [i]); l; l = l -> next) {
 			    print_lease (l);
 		    }
@@ -2226,12 +2412,14 @@ void dump_subnets ()
 	}
 }
 
-HASH_FUNCTIONS (lease, const unsigned char *, struct lease, lease_hash_t,
-		lease_reference, lease_dereference)
+HASH_FUNCTIONS(lease_ip, const unsigned char *, struct lease, lease_ip_hash_t,
+	       lease_reference, lease_dereference, do_ip4_hash)
+HASH_FUNCTIONS(lease_id, const unsigned char *, struct lease, lease_id_hash_t,
+	       lease_reference, lease_dereference, do_id_hash)
 HASH_FUNCTIONS (host, const unsigned char *, struct host_decl, host_hash_t,
-		host_reference, host_dereference)
+		host_reference, host_dereference, do_string_hash)
 HASH_FUNCTIONS (class, const char *, struct class, class_hash_t,
-		class_reference, class_dereference)
+		class_reference, class_dereference, do_string_hash)
 
 #if defined (DEBUG_MEMORY_LEAKAGE) && \
 		defined (DEBUG_MEMORY_LEAKAGE_ON_EXIT)
@@ -2251,7 +2439,7 @@ extern int end;
 extern struct lease *lease_hunks;
 #endif
 
-void free_everything ()
+void free_everything(void)
 {
 	struct subnet *sc = (struct subnet *)0, *sn = (struct subnet *)0;
 	struct shared_network *nc = (struct shared_network *)0,
@@ -2262,12 +2450,8 @@ void free_everything ()
 		*in = (struct interface_info *)0;
 	struct class *cc = (struct class *)0, *cn = (struct class *)0;
 	struct collection *lp;
-	void *st = (shared_networks
-		    ? (shared_networks -> next
-		       ? shared_networks -> next -> next : 0) : 0);
 	int i;
 
-
 	/* Get rid of all the hash tables. */
 	if (host_hw_addr_hash)
 		host_free_hash_table (&host_hw_addr_hash, MDL);
@@ -2276,13 +2460,13 @@ void free_everything ()
 		host_free_hash_table (&host_uid_hash, MDL);
 	host_uid_hash = 0;
 	if (lease_uid_hash)
-		lease_free_hash_table (&lease_uid_hash, MDL);
+		lease_id_free_hash_table (&lease_uid_hash, MDL);
 	lease_uid_hash = 0;
 	if (lease_ip_addr_hash)
-		lease_free_hash_table (&lease_ip_addr_hash, MDL);
+		lease_ip_free_hash_table (&lease_ip_addr_hash, MDL);
 	lease_ip_addr_hash = 0;
 	if (lease_hw_addr_hash)
-		lease_free_hash_table (&lease_hw_addr_hash, MDL);
+		lease_id_free_hash_table (&lease_hw_addr_hash, MDL);
 	lease_hw_addr_hash = 0;
 	if (host_name_hash)
 		host_free_hash_table (&host_name_hash, MDL);
@@ -2379,7 +2563,15 @@ void free_everything ()
 	}
 
 	/* So are shared networks. */
+	/* XXX: this doesn't work presently, but i'm ok just filtering
+	 * it out of the noise (you get a bigger spike on the real leaks).
+	 * It would be good to fix this, but it is not a "real bug," so not
+	 * today.  This hack is incomplete, it doesn't trim out sub-values.
+	 */
 	if (shared_networks) {
+		shared_network_dereference (&shared_networks, MDL);
+	/* This is the old method (tries to free memory twice, broken) */
+	} else if (0) {
 	    shared_network_reference (&nn, shared_networks, MDL);
 	    do {
 		if (nn) {
@@ -2395,8 +2587,8 @@ void free_everything ()
 		if (nc -> pools) {
 		    pool_reference (&pn, nc -> pools, MDL);
 		    do {
-			struct lease **lptr [5];
-			
+			struct lease **lptr[RESERVED_LEASES+1];
+
 			if (pn) {
 			    pool_reference (&pc, pn, MDL);
 			    pool_dereference (&pn, MDL);
@@ -2405,15 +2597,16 @@ void free_everything ()
 			    pool_reference (&pn, pc -> next, MDL);
 			    pool_dereference (&pc -> next, MDL);
 			}
-			
+
 			lptr [FREE_LEASES] = &pc -> free;
 			lptr [ACTIVE_LEASES] = &pc -> active;
 			lptr [EXPIRED_LEASES] = &pc -> expired;
 			lptr [ABANDONED_LEASES] = &pc -> abandoned;
 			lptr [BACKUP_LEASES] = &pc -> backup;
+			lptr [RESERVED_LEASES] = &pc->reserved;
 
 			/* As (sigh) are leases. */
-			for (i = 0; i < 5; i++) {
+			for (i = FREE_LEASES ; i <= RESERVED_LEASES ; i++) {
 			    if (*lptr [i]) {
 				lease_reference (&ln, *lptr [i], MDL);
 				do {
@@ -2474,14 +2667,21 @@ void free_everything ()
 
 	universe_free_hash_table (&universe_hash, MDL);
 	for (i = 0; i < universe_count; i++) {
+#if 0
 		union {
 			const char *c;
 			char *s;
 		} foo;
+#endif
 		if (universes [i]) {
-			if (universes [i] -> hash)
-			    option_free_hash_table (&universes [i] -> hash,
-						    MDL);
+			if (universes[i]->name_hash)
+			    option_name_free_hash_table(
+						&universes[i]->name_hash,
+						MDL);
+			if (universes[i]->code_hash)
+			    option_code_free_hash_table(
+						&universes[i]->code_hash,
+						MDL);
 #if 0
 			if (universes [i] -> name > (char *)&end) {
 				foo.c = universes [i] -> name;
@@ -2500,7 +2700,9 @@ void free_everything ()
 	relinquish_free_binding_values ();
 	relinquish_free_option_caches ();
 	relinquish_free_packets ();
+#if defined(COMPACT_LEASES)
 	relinquish_lease_hunks ();
+#endif
 	relinquish_hash_bucket_hunks ();
 	omapi_type_relinquish ();
 }
